<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit ; // Exit if accessed directly.
}

if ( ! class_exists( 'SUMOCategoryProductPricing' ) ) {

	// Initialize Discounts Rule Tab

	class SUMOCategoryProductPricing {

		// Construct the Class Name

		public function __construct() {

			add_action( 'admin_init', array( $this, 'sumo_pricing_default_settings' ), 103 ) ; // call the init function to update the default settings on page load
			// make it appear in Discount System Discounts Rule
			add_action( 'woocommerce_sp_settings_tabs_array', array( $this, 'initialize_tab' ) ) ;

			// Initialize Admin Fields in Discounts Rule
			add_action( 'woocommerce_sp_settings_tabs_sumo_cat_pro_pricing', array( $this, 'initialize_visual_appearance_admin_fields' ) ) ;

			// Initialize Update Fields in Discounts Rule
			add_action( 'woocommerce_update_options_sumo_cat_pro_pricing', array( $this, 'update_data_from_admin_fields' ) ) ;


			// Admin Enqueue Scripts for the Discounts Rule
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) ) ;

			// Add Custom Type of Field Option
			add_action( 'woocommerce_admin_field_sumo_cat_pro_pricing_rule', array( $this, 'sumo_cat_pro_pricing_rule' ) ) ;

			add_action( 'wp_ajax_sumo_pricing_rule_sortable_for_cat_pro', array( $this, 'admin_request_from_ajax_sortable' ) ) ;

			add_action( 'wp_ajax_sumo_pricing_uniqid_for_cat_pro', array( $this, 'ajax_function_response_alteration_html' ) ) ;

			add_action( 'woocommerce_update_options_sumo_cat_pro_pricing', array( $this, 'save_data_pricing_rule' ) ) ;
		}

		// Declare the Discounts Rule Tab in Discount System

		public static function initialize_tab( $settings_tab ) {
			if ( ! is_array( $settings_tab ) ) {
				$settings_tab = ( array ) $settings_tab ;
			}
			$settings_tab[ 'sumo_cat_pro_pricing' ] = __( 'Category/Product Discounts', 'sumodiscounts' ) ;
			return array_filter( $settings_tab ) ;
		}

		// Initialize Admin Fields in Discount System

		public static function initialize_admin_fields() {
			global $woocommerce ;
			return apply_filters( 'woocommerce_cat_pro_pricing_rule', array(
				array(
					'name' => __( 'Category/Product Discounts', 'sumodiscounts' ),
					'type' => 'title',
					'id'   => '_sp_pricing_rule_settings'
				),
				array(
					'name'    => __( 'Rule Priority', 'sumodiscounts' ),
					'type'    => 'select',
					'id'      => 'sumo_cat_pro_pricing_priority_settings',
					'newids'  => 'sumo_cat_pro_pricing_priority_settings',
					'class'   => 'sumo_cat_pro_pricing_priority_settings',
					'options' => array(
						'1' => __( 'First Matched Rule', 'sumodiscounts' ),
						'2' => __( 'Last Matched Rule', 'sumodiscounts' ),
						'3' => __( 'Minimum Discount', 'sumodiscounts' ),
						'4' => __( 'Maximum Discount', 'sumodiscounts' )
					),
				),
				array(
					'type' => 'sectionend',
					'id'   => '_sp_pricing_rule_settings'
				),
				array(
					'type' => 'sumo_cat_pro_pricing_rule'
				),
					) ) ;
		}

		// Make it appear visually in Discount System

		public static function initialize_visual_appearance_admin_fields() {
			woocommerce_admin_fields( self::initialize_admin_fields() ) ;
		}

		// Update the Settings of Discount System

		public static function update_data_from_admin_fields( $post ) {
			woocommerce_update_options( self::initialize_admin_fields() ) ;
		}

		/**
		 * Initialize the Default Settings by looping this function
		 */
		public static function sumo_pricing_default_settings() {
			global $woocommerce ;
			foreach ( self::initialize_admin_fields() as $setting ) {
				if ( isset( $setting[ 'newids' ] ) && isset( $setting[ 'std' ] ) ) {
					add_option( $setting[ 'newids' ], $setting[ 'std' ] ) ;
				}
			}
		}

		// Enqueue Script

		public static function admin_enqueue_scripts() {
			wp_enqueue_script( 'jquery' ) ;
			wp_enqueue_script( 'jquery-ui-sortable' ) ;
			wp_enqueue_script( 'jquery-ui-datepicker' ) ;
		}

		// Declare the Options

		public static function sumo_cat_pro_pricing_rule() {
			$actionforsorting     = 'sumo_pricing_rule_sortable_for_cat_pro' ;
			$actionforaddrule     = 'sumo_pricing_uniqid_for_cat_pro' ;
			$classforaddlocalrule = 'add_new_row_for_cat_pro' ;
			$get_saved_data       = get_option( 'sumo_pricing_rule_fields_for_cat_pro' ) ;
			$pricingtype          = 'cat_pro_pricing' ;
			$array                = array(
				'actionforsorting'     => $actionforsorting,
				'actionforaddrule'     => $actionforaddrule,
				'classforaddlocalrule' => $classforaddlocalrule,
				'get_saved_data'       => $get_saved_data,
				'pricing_type'         => $pricingtype,
				'nameforinputfield'    => 'sumo_pricing_rule_fields_for_cat_pro'
					) ;
			fpsd_to_add_settings_for_rule( $array ) ;
		}

		public static function admin_request_from_ajax_sortable() {
			check_ajax_referer( 'sumo-discounts-sorting-nonce', 'sumo_discount_security' ) ;

			update_option( 'sumo_dynamic_pricing_drag_position_for_cat_pro', isset( $_REQUEST[ 'data' ] ) ? wc_clean( wp_unslash( $_REQUEST[ 'data' ] ) ) : array()  ) ;
		}

		// Save the Fields

		public static function save_data_pricing_rule( $value ) {
			$sumo_pricing_rule_fields_for_cat_pro = isset( $_REQUEST[ 'sumo_pricing_rule_fields_for_cat_pro' ] ) ? wc_clean( wp_unslash( $_REQUEST[ 'sumo_pricing_rule_fields_for_cat_pro' ] ) ) : array() ;
			update_option( 'sumo_pricing_rule_fields_for_cat_pro', $sumo_pricing_rule_fields_for_cat_pro ) ;
		}

		// Sort Div with Sortable Manner
		public static function ajax_function_response_alteration_html() {
			if ( isset( $_REQUEST ) ) {
				$phpuniqid = uniqid() ;
				$uniqid    = isset( $_REQUEST[ 'uniq_id' ] ) ? wc_clean( wp_unslash( $_REQUEST[ 'uniq_id' ] ) ) : '' ;
				$new_array = array(
					'unique_id'         => $uniqid,
					'phpunique_id'      => $phpuniqid,
					'nameforfields'     => 'sumo_cat_pro_rule',
					'rule_type'         => 'cat_pro_pricing',
					'nameforinputfield' => 'sumo_pricing_rule_fields_for_cat_pro'
						) ;
				echo do_shortcode( fpsd_array_to_field_conversion( $new_array ) ) ;
			}
			exit() ;
		}

	}

	new SUMOCategoryProductPricing() ;
}  
